@@ -1,14 +1,17 @@
 # -*- coding: utf-8 -*-
+from __future__ import absolute_import
+from __future__ import with_statement
 
 import sys
 import code
+import string
 import getpass
 import inspect
 import warnings
 
 import argparse
 
-from flask import Flask
+from flask import Flask, _request_ctx_stack
 
 __all__ = ["Command", "Shell", "Server", "Manager", "Option",
            "prompt", "prompt_pass", "prompt_bool", "prompt_choices"]
@@ -51,7 +54,7 @@ def prompt_pass(name, default=None):
             return default
 
 
-def prompt_bool(name, default=False):
+def prompt_bool(name, default=False, yes_choices=None, no_choices=None):
     
     """
     Grabs user input from command line and converts to boolean
@@ -59,40 +62,56 @@ def prompt_bool(name, default=False):
 
     :param name: prompt text
     :param default: default value if no input provided.
+    :param yes_choices: default 'y', 'yes', '1', 'on', 'true', 't'
+    :param no_choices: default 'n', 'no', '0', 'off', 'false', 'f'
     """
+
+    yes_choices = yes_choices or ('y', 'yes', '1', 'on', 'true', 't')
+    no_choices = no_choices or ('n', 'no', '0', 'off', 'false', 'f')
     
     while True:
-        rv = prompt(name + '?', default and 'Y' or 'N')
+        rv = prompt(name + '?', default and yes_choices[0] or no_choices[0])
         if not rv:
             return default
-        if rv.lower() in ('y', 'yes', '1', 'on', 'true', 't'):
+        if rv.lower() in yes_choices:
             return True
-        elif rv.lower() in ('n', 'no', '0', 'off', 'false', 'f'):
+        elif rv.lower() in no_choices:
             return False
 
 
-def prompt_choices(name, choices, default=None):
+def prompt_choices(name, choices, default=None, 
+    resolve=string.lower, no_choice=('none',)):
     
     """
     Grabs user input from command line from set of provided choices.
 
     :param name: prompt text
-    :param choices: list or tuple of available choices
+    :param choices: list or tuple of available choices. Choices may be 
+                    single strings or (key, value) tuples.
     :param default: default value if no input provided.
+    :param no_choice: acceptable list of strings for "null choice"
     """
+    
+    _choices = []
+    options = []
+    
+    for choice in choices:
+        if isinstance(choice, basestring):
+            options.append(choice)
+        else:
+            options.append("%s [%s]" % (choice[1], choice[0]))
+            choice = choice[0]
+        _choices.append(choice)
 
-    if default is None:
-        default = choices[0]
     while True:
-        rv = prompt(name + '? - (%s)' % ', '.join(choices), default)
-        rv = rv.lower()
+        rv = prompt(name + '? - (%s)' % ', '.join(options), default)
         if not rv:
             return default
-        if rv in choices:
-            if rv == 'none':
-                return None
-            else:
-                return rv
+        rv = resolve(rv)
+        if rv in no_choice:
+            return None
+        if rv in _choices:
+            return rv
 
 
 class Option(object):
@@ -164,7 +183,7 @@ class Command(object):
         
         return parser
 
-    def run(self, app):
+    def run(self):
 
         """
         Runs a command. This must be implemented by the subclass. The first
@@ -205,15 +224,18 @@ class Shell(Command):
 
     :param banner: banner appearing at top of shell when started
     :param make_context: a callable returning a dict of variables 
-    used in the shell namespace. The callable takes a single argument,
-    "app", the Flask instance. By default returns a dict consisting
-    of just the app.
-    :param use_ipython: use IPython shell if available, ignore if not.
-    The IPython shell can be turned off in command line by passing the
-    --no-ipython flag.
+                         used in the shell namespace. The callable 
+                         takes a single argument, "app", the Flask 
+                         instance. By default returns a dict consisting
+                         of just the app.
+    :param use_ipython: use IPython shell if available, ignore if not. 
+                        The IPython shell can be turned off in command 
+                        line by passing the **--no-ipython** flag.
     """
 
     banner = ''
+
+    description = 'Runs a Python shell inside Flask application context.'
     
     def __init__(self, banner=None, make_context=None, use_ipython=True):
 
@@ -222,10 +244,10 @@ class Shell(Command):
         self.use_ipython = use_ipython
 
         if make_context is None:
-            make_context = lambda app: dict(app=app)
+            make_context = lambda: dict(app=_request_ctx_stack.top.app)
 
         self.make_context = make_context
-
+    
     def get_options(self):
 
         return (
@@ -234,22 +256,22 @@ class Shell(Command):
                        dest='no_ipython',
                        default=not(self.use_ipython)),)
 
-    def get_context(self, app):
+    def get_context(self):
         
         """
         Returns a dict of context variables added to the shell namespace.
         """
 
-        return self.make_context(app)
+        return self.make_context()
 
-    def run(self, app, no_ipython):
+    def run(self, no_ipython):
 
         """
         Runs the shell. Unless no_ipython is True or use_python is False
         then runs IPython shell if that is installed.
         """
 
-        context = self.get_context(app)
+        context = self.get_context()
         if not no_ipython:
             try:
                 import IPython
@@ -270,11 +292,16 @@ class Server(Command):
     :param host: server host
     :param port: server port
     :param use_debugger: if False, will no longer use Werkzeug debugger.
-    This can be overriden in the command line by passing the -d flag.
-    :param use_reloader: if Flase, will no loner use auto-reloader.
-    This can be overriden in the command line by passing the -r flag.
+                         This can be overriden in the command line 
+                         by passing the **-d** flag.
+    :param use_reloader: if False, will no longer use auto-reloader.
+                         This can be overriden in the command line by 
+                         passing the **-r** flag.
+                         
     """
 
+    description = 'Runs the Flask development server i.e. app.run()'
+
     def __init__(self, host='127.0.0.1', port=5000, use_debugger=True,
         use_reloader=True):
 
@@ -306,7 +333,8 @@ class Server(Command):
                        dest='use_reloader',
                        default=self.use_reloader))
 
-    def run(self, app, host, port, use_debugger, use_reloader):
+    def run(self, host, port, use_debugger, use_reloader):
+        app = _request_ctx_stack.top.app
         app.run(host=host,
                 port=port,
                 debug=use_debugger,
@@ -437,7 +465,6 @@ class Manager(object):
 
         # first arg is always "app" : ignore
 
-        args = args[1:]
         defaults = defaults or []
         kwargs = dict(zip(*[reversed(l) for l in (args, defaults)]))
 
@@ -588,8 +615,8 @@ class Manager(object):
         app = self.create_app(**app_namespace.__dict__)
 
         with app.test_request_context():
-            command.run(app, **command_namespace.__dict__)
-        
+            command.run(**command_namespace.__dict__)
+
     def run(self, commands=None):
         
         """
